use std::fmt::Write;
use dim_sdk::{plugin_info, Context, DimPlugin, Message, Result};

plugin_info!(
    Plug,                    // Your main global structs name that implements `DimPlugin`
    "example_rust",          // Plugin name
    "0.0.0",                 // Plugin Version (leave empty for none)
    "GPLv3"                  // Plugin license (leave empty for none)
    );

#[derive(Debug, Clone)]
struct Plug {
    ctx: Context,
    counter: usize,
}

impl DimPlugin for Plug {
    fn init(ctx: Context) -> dim_sdk::Result<Self> {
        // Initialise data, this will run once, it will not run again after reload
        // log::info!("hewo from rust :3");
        Ok(Self {
            ctx,
            counter: 0
        })
    }
    fn on_message(&mut self, msg: Message) -> dim_sdk::Result<()> {
        self.ctx.buf.reset();
        match msg.name.as_str() {
            "poll" => {
                // Write to buffer the text you want to display, keep this short
                write!(self.ctx, "Hello from rust! ({})", self.counter)?;
                self.counter += 1;
            }
            _ => ()
        }

        Ok(())
    }
    fn pre_reload(&mut self) -> Result<()> {
        // Do stuff before reload, probably save important things because theres a good chance
        // (especially on rust) that if you change the data layout it will die
        Ok(())
    }
    fn post_reload(&mut self) -> Result<()>{
        // Do stuff after reloading plugin, state a.k.a this struct has the same data, will crash
        // if the data layout changed
        Ok(())
    }
    fn free(&mut self) -> Result<()> {
        // Yout probably dont need this but its for freeing things before the plugin gets unloaded
        Ok(())
    }
}




