use std::{ffi::{c_char, CStr}, path::PathBuf};

use crate::CBuffer;


#[derive(Debug, Clone)]
#[repr(C)]
pub struct ContextRaw {
    pub config_dir: *const c_char,
    pub funcs: PluginContextFuncs,
    pub buf: *mut u8,
}

#[derive(Debug, Clone)]
pub struct Context {
    pub config_dir: PathBuf,
    pub funcs: PluginContextFuncs,
    pub buf: CBuffer
}

#[repr(C)]
#[derive(Debug, Clone)]
pub struct PluginContextFuncs {
    pub log: unsafe extern "C" fn(module: *const c_char, level: log::Level, s: *const c_char),
}

impl Context {
    pub fn new(cr: ContextRaw) -> Self {
        let config_dir = unsafe {
            let v = CStr::from_ptr(cr.config_dir);
            PathBuf::from(v.to_string_lossy().to_string().clone())
        };
        Self {
            config_dir,
            funcs: cr.funcs.clone(),
            buf: CBuffer::from_raw_parts_mut(cr.buf, crate::BUF_SZ)
        }
    }
}

impl std::fmt::Write for Context {
    fn write_str(&mut self, buf: &str) -> Result<(), std::fmt::Error> { 
        self.buf.write_str(buf)
    }
}

