
#[macro_use]
mod magic;
mod c_buffer;
mod plugin_info;
mod context;
mod logger;

use std::ffi::{c_char, CStr};

pub use c_buffer::*;
pub use plugin_info::*;
pub use anyhow::Result;
pub use context::*;
pub use logger::*;

pub const BUF_SZ: usize = 2048;

pub trait DimPlugin: Clone{
    fn init(ctx: Context) -> Result<Self>;
    fn pre_reload(&mut self) -> Result<()>;
    fn post_reload(&mut self) -> Result<()>;
    fn on_message(&mut self, msg: Message) -> Result<()>;
    fn free(&mut self) -> Result<()>;
}

#[derive(Debug)]
pub struct Message {
    pub name: String,
    pub value: String,
}

#[derive(Debug, Clone)]
#[repr(C)]
pub struct MessageRaw {
    pub name: *const c_char,
    pub value: *const c_char,
}


impl Message {
    pub fn new(msg: MessageRaw) -> Self {
        unsafe {
            Self { 
                name: CStr::from_ptr(msg.name.clone())
                        .to_string_lossy().to_string(), 
                value: CStr::from_ptr(msg.value.clone())
                        .to_string_lossy().to_string()
            }
        }
    }

}





