use std::ffi::{c_char, CString};

use crate::Context;

#[allow(dead_code)]
#[derive(Debug, Clone)]
pub struct DimLogger {
    log_fn:  unsafe extern "C" fn(module: *const c_char, level: log::Level, s: *const c_char),
}

impl log::Log for DimLogger {
    fn enabled(&self, _: &log::Metadata) -> bool {true}

    fn log(&self, record: &log::Record) {
        unsafe {
            let module = format!("(plug) {}", record.module_path().unwrap_or("UNKNOWN"));
            let module = CString::new(module).unwrap();
            let s = CString::new(format!("{}", record.args())).unwrap();
            (self.log_fn)(module.as_ptr(), record.level(), s.as_ptr());
        }
    }

    fn flush(&self) {}
}

impl DimLogger {
    pub fn new(ctx: &Context) -> Self {
        Self {
            log_fn: ctx.funcs.log,
        }
    }

    pub fn init(&self) {
        let _ = log::set_boxed_logger(Box::new(self.clone()))
            .map(|()| log::set_max_level(log::LevelFilter::Debug));
    }
}
