// Lord have mercy on me
#[macro_export]
macro_rules! plugin_info {
    ($typ:ty, $name:literal, $version:literal, $license:literal) => {
         
        lazy_static::lazy_static!(
            static ref PLUGIN_INFO: $crate::PluginInfo = $crate::PluginInfo::new($name, $version, $license);
        );

        static mut PLUG: Option<$typ> = None as Option<$typ>;

        #[no_mangle]
        unsafe extern "C" fn plug_get_info() -> *const std::ffi::c_void {
            PLUGIN_INFO.get_raw_ptr()
        }

        impl $typ {
            pub fn get_plugin_info(&self) -> $crate::PluginInfo {
                $crate::PluginInfo::new(
                    PLUGIN_INFO.name().as_str(),
                    PLUGIN_INFO.version().as_str(),
                    PLUGIN_INFO.license().as_str()
                    )
            }
        }

        #[no_mangle]
        unsafe extern "C" fn plug_init(ctx: *const $crate::ContextRaw) -> i32 {
            let ctx = $crate::Context::new((*ctx).clone());
            $crate::DimLogger::new(&ctx).init();
            match <$typ>::init(ctx) {
                Ok(v) => {
                    PLUG = Some(v);
                    return 0;
                }
                Err(e) => {
                    log::error!("Had an error while initialising: {e}");
                    return 1;
                }
            };
        }

        #[no_mangle]
        unsafe extern "C" fn plug_pre_reload() -> *mut $typ {
            //TODO: Untested 
            if let Some(p) = &mut PLUG {
                if let Err(e) = p.pre_reload() {
                    log::error!("Had an error on pre reload: {e}");
                    return std::ptr::null_mut();
                }
                return p as *mut $typ;
            }
            unreachable!();
        }

        #[no_mangle]
        unsafe extern "C" fn plug_post_reload(state: *mut $typ) -> i32 {
            PLUG = Some((*state).clone());

            if let Some(p) = &mut PLUG {
                if let Err(e) = p.post_reload() {
                    log::error!("Had an error on pre reload: {e}");
                    return 1;
                }
            } else {
                unreachable!();
            }
            //TODO: Untested
            return 0;
        }

        #[no_mangle]
        unsafe extern "C" fn plug_on_msg(msg: *const $crate::MessageRaw) -> i32 {
            let msg = $crate::Message::new((*msg).clone());
            if let Some(p) = &mut PLUG {
                if let Err(e) = p.on_message(msg) {
                    log::error!("Had error on `on_msg`: {e}");
                    return 1;
                }
            } else {
                unreachable!();
            }
            return 0;
        }

        #[no_mangle]
        unsafe extern "C" fn plug_free() -> i32 {
            if let Some(p) = &mut PLUG {
                // std::alloc::dealloc(PLUG as *mut u8, std::alloc::Layout::new::<$typ>());
                if let Err(e) = p.free() {
                    log::error!("Had error on `free`: {e}");
                    return 1;
                }
            } else {
                unreachable!();
            }
            return 0;
        }

    };
}
