use clap::{Parser, Subcommand};


#[derive(Debug, Parser)]
pub struct CliArgs {
    /// Directory where all *.dim plugin files are stored
    #[arg(long, short, default_value="./plugins")]
    pub plugin_dir: camino::Utf8PathBuf,
    
    /// Directory where all configurations are stored
    #[arg(long, short, default_value="./config")]
    pub config_dir: camino::Utf8PathBuf,
    
    /// Print more debug info
    #[arg(long, short)]
    pub debug: bool,
    
    /// Socket path
    #[arg(long, short)]
    pub sock_path: Option<camino::Utf8PathBuf>,

    #[command(subcommand)]
    pub subcommand: Option<CliSubcommand>,
}

#[derive(Debug, Subcommand)]
pub enum CliSubcommand {
    /// Send a message through ipc to the running DIM instance
    Send {
        /// Command name, eg. `clock:reset`, 
        /// `clock:` will be stripped and the command will be sent
        /// only to plugin `clock`, dont include it to send it to all plugins
        #[arg(long, short)]
        name: String,
        
        /// Value to send
        #[arg(long, short, default_value="")]
        value: String,
    },

    /// Reload one or all plugins
    Reload {
        /// (Optional) Which plugin to Reload
        name: Option<String>
    }
}
