use std::{collections::HashMap, ffi::{CString, NulError}};

use x11::xlib;

use crate::{config::Config, plugman::PolledText};



pub struct Display {
    window: u64,
    display: *mut xlib::Display

}

impl Display {
    pub fn new() -> Result<Self, ()> {
        let (window, display) = unsafe { 
            let display = xlib::XOpenDisplay(std::ptr::null());
            
            if display.is_null() {
                log::error!("Could not open x11 display");
                return Err(());
            }

            (
                xlib::XDefaultRootWindow(display),
                display
            )
        };

        Ok(Self {
            window,
            display
        })
    }
    
    pub fn write_display_name(&mut self, name: &str) -> Result<(), NulError> {
        let cstr = CString::new(name)?;
        unsafe {
            xlib::XStoreName(self.display, self.window, cstr.as_ptr() as *const i8);
            xlib::XFlush(self.display);
        }

        Ok(())
    }

    pub fn write_with_vec(&mut self, cfg: &Config, v: HashMap<String, PolledText>) -> Result<(), NulError> {
        let mut buf = Vec::new();
        
        let mut v = if cfg.plugins.as_whitelist {
            let mut vv = HashMap::new();
            for (n, pt) in v {
                if cfg.plugins.blacklist.contains(&n) {
                    vv.insert(n, pt);
                }
            }
            vv
        } else {
            let mut vv = HashMap::new();
            for (n, pt) in v {
                if !cfg.plugins.blacklist.contains(&n) {
                    vv.insert(n, pt);
                }
            }
            vv
        };

        let mut used_values = Vec::new();
        for tv in &cfg.plugins.template {
            let mut found = false;
            for (pname, pt) in &v {
                if *pname == *tv {
                    if !pt.disabled() {
                        buf.push(pt.text().clone());
                    }

                    used_values.push(pname.clone());
                    found = true;
                    break;
                }
            }
            if !found {
                // log::warn!("Plugin '{}' was not found even though it was included in the template", tv);
            }
        }
        
        for val in used_values {
            v.remove(&val);
        }
        
        let mut kv: Vec<(&String, &PolledText)> = v.iter().collect();
        kv.sort_by(|a, b| a.0.cmp(b.0));

        for (_, pt) in &kv {
            buf.push(pt.text().clone());
        }

        let text = buf.join(&cfg.seperator);

        self.write_display_name(&text)?;

        Ok(()) 
    }
}

