use std::ffi::{c_char, CString};

pub mod client;
pub mod server;

#[derive(Debug)]
pub struct Message {
    pub plugin: Option<String>,
    pub name: CString,
    pub value: CString,
    pub ffi: MessageFFI
}

#[derive(Debug)]
#[repr(C)]
pub struct MessageFFI {
    pub name: *const c_char,
    pub value: *const c_char,
}

impl Message {
    pub fn from_string_kv<S: ToString>(name: S, value: S) -> Self {
        let mut name = name.to_string();
        let value = value.to_string();
        let plugin;
        if name.contains(':') {
            let ns: Vec<&str> = name.split(':').collect();
            plugin = Some(ns[0].to_string());
            name = ns[1..].join(":");

        } else {
            plugin = None;
        }
           
        let name = CString::new(name).unwrap();
        let value = CString::new(value).unwrap();


        Self {
            ffi: MessageFFI {
                name: name.as_ptr(),
                value: value.as_ptr()
            },
            plugin, 
            name, 
            value,
        }
    }

    pub fn from_string<S: ToString>(k: S) -> Self {
        let k = k.to_string();
        let mut v = String::new();
        let mut name = k.clone();
        if k.contains('=') {
            let kv: Vec<&str> = k.split('=').collect();
            name = kv[0].to_string();
            v = kv[1..].join("=");
        }

        Self::from_string_kv(name, v)
    }

}
