#![allow(internal_features)]
#![feature(core_intrinsics)]

use std::{process::ExitCode, time::Duration};
use clap::Parser;

mod display;
mod plugman;
mod config;
mod util;
mod cli;
mod logger;
mod ipc;

// TODO: Make function available in ctx that disables the plugin
// TODO: Set up ipc with unix sockets
// TODO: Allow sending messages command -> running DIM instance -> plugin with ipc
// TODO: Clickable bar: https://dwm.suckless.org/patches/statuscmd/
// TODO: Run code through clippy
fn main() -> ExitCode {
    let ca = cli::CliArgs::parse();
    
    logger::init(&ca);

    let config;
    match config::Config::new(ca) {
        Ok(v) => config = v,
        Err(e) => {
            log::error!("Failed to parse config: {e:?}");
            return ExitCode::from(2);
        }
    };
    let Ok(mut disp) = display::Display::new() else {
        return ExitCode::from(1);
    }; 
    

    let mut pm = plugman::PlugMan::new(1024); // idk tbh
    if let Err(e) = pm.load(&config.plugins.path){
        log::error!("Failed to load plugins: {e:?}");
        return ExitCode::from(3);
    }

    pm.init_plugins(&config);
    
    loop {
        let vals = pm.poll_plugins();

        log::debug!("vals: {vals:?}");
        if let Err(e) = disp.write_with_vec(&config, vals) {
            log::warn!("Failed to write too bar: {e}");
        }
        std::thread::sleep(Duration::from_millis(250));
    }
}
