use std::ffi::{c_char, CStr, CString};
use super::Plugin;

pub struct PluginContextContainer {
    pub inner: PluginContext,
    _config_dir: CString
}


#[repr(C)]
pub struct PluginContext {
    pub config_dir: *const c_char,
    funcs: PluginContextFuncs,
    buf: *mut u8
}

#[repr(C)]
pub struct PluginContextFuncs {
    log: unsafe extern "C" fn(module: *const c_char, level: log::Level, s: *const c_char),
}

impl PluginContext {
    pub fn new(plug: &Plugin, config_dir: &CString) -> Self {
        Self {
            config_dir: config_dir.as_ptr(),
            funcs: PluginContextFuncs {
                log: plug_log,
            },
            buf: plug.buf
        }
    }
}


impl PluginContextContainer {
    pub fn new(plug: &Plugin, config_dir: &String) -> Self {
        let _config_dir = CString::new(config_dir.clone()).unwrap();
        Self {
            inner: PluginContext::new(plug, &_config_dir),
            _config_dir,
        }
    }
}

pub unsafe extern "C" fn plug_log(module: *const c_char, level: log::Level, s: *const c_char) {
    let module = CStr::from_ptr(module).to_string_lossy().to_string();
    let s = CStr::from_ptr(s).to_string_lossy().to_string();

    log::log!(target: &module, level, "{s}");
}
